@@ -14,10 +14,12 @@
 # limitations under the License.
 """Globbing utility module."""
 
-from enum import Enum
 import itertools
 import re
+
 import six
+from enum import Enum
+import biggraphite.metric as bg_metric
 
 # http://graphite.readthedocs.io/en/latest/render_api.html#paths-and-wildcards
 _GRAPHITE_GLOB_RE = re.compile(r"^[^*?{}\[\]]+$")
@@ -49,14 +51,14 @@ def _is_valid_glob(glob):
     """
     depth = 0
     for c in glob:
-        if c == '{':
+        if c == "{":
             depth += 1
-        elif c == '}':
+        elif c == "}":
             depth -= 1
             if depth < 0:
                 # Mismatched braces
                 return False
-        elif c == '.':
+        elif c == ".":
             if depth > 0:
                 # Component separator in the middle of a group
                 return False
@@ -92,7 +94,7 @@ def tokenize(glob):
     Returns:
       Iterator on a token stream.
     """
-    SPECIAL_CHARS = '.?*[-]{,}'
+    SPECIAL_CHARS = ".?*[-]{,}"
     is_escaped = False
     is_char_select = False
     tmp = ""
@@ -107,10 +109,10 @@ def tokenize(glob):
             tmp += c
             is_escaped = False
             continue
-        elif c == '\\':
+        elif c == "\\":
             is_escaped = True
             continue
-        elif c not in SPECIAL_CHARS or (c == '-' and not is_char_select):
+        elif c not in SPECIAL_CHARS or (c == "-" and not is_char_select):
             if token and token != TokenType.LITERAL:
                 yield token, None
                 token, tmp = TokenType.LITERAL, ""
@@ -122,35 +124,35 @@ def tokenize(glob):
             token, tmp = None, ""
 
         # Special chars handling
-        if c == '.':
+        if c == ".":
             yield TokenType.PATH_SEPARATOR, ""
-        elif c == '?':
+        elif c == "?":
             yield TokenType.WILD_CHAR, ""
-        elif c == '*':
+        elif c == "*":
             # Look-ahead for wild path (globstar)
-            if i + 1 < len(glob) and glob[i + 1] == '*':
+            if i + 1 < len(glob) and glob[i + 1] == "*":
                 i += 1
                 yield TokenType.WILD_PATH, ""
             else:
                 yield TokenType.WILD_SEQUENCE, ""
-        elif c == '[':
+        elif c == "[":
             is_char_select = True
             # Look-ahead for negated selector (not in)
-            if i + 1 < len(glob) and glob[i + 1] == '!':
+            if i + 1 < len(glob) and glob[i + 1] == "!":
                 i += 1
                 yield TokenType.CHAR_SELECT_NEGATED_BEGIN, ""
             else:
                 yield TokenType.CHAR_SELECT_BEGIN, ""
-        elif c == '-':
+        elif c == "-":
             yield TokenType.CHAR_SELECT_RANGE_DASH, ""
-        elif c == ']':
+        elif c == "]":
             is_char_select = False
             yield TokenType.CHAR_SELECT_END, ""
-        elif c == '{':
+        elif c == "{":
             yield TokenType.EXPR_SELECT_BEGIN, ""
-        elif c == ',':
+        elif c == ",":
             yield TokenType.EXPR_SELECT_SEPARATOR, ""
-        elif c == '}':
+        elif c == "}":
             yield TokenType.EXPR_SELECT_END, ""
         else:
             raise Exception("Unexpected character '%s'" % c)
@@ -160,7 +162,7 @@ def tokenize(glob):
         yield token, tmp
 
 
-def _glob_to_regex(glob):
+def glob_to_regex(glob):
     """Convert a Graphite globbing pattern into a regular expression.
 
     This function does not check for glob validity, if you want usable regexes
@@ -181,7 +183,7 @@ def _glob_to_regex(glob):
     ans = ""
     for token, data in tokenize(glob):
         if token == TokenType.PATH_SEPARATOR:
-            ans += re.escape('.')
+            ans += re.escape(".")
         elif token == TokenType.LITERAL:
             ans += re.escape(data)
         elif token == TokenType.WILD_CHAR:
@@ -205,9 +207,8 @@ def _glob_to_regex(glob):
         elif token == TokenType.EXPR_SELECT_END:
             ans += ")"
         else:
-            raise Exception(
-                "Unexpected token type '%s' with data '%s'" % (token, data))
-    return '^' + ans + '$'
+            raise Exception("Unexpected token type '%s' with data '%s'" % (token, data))
+    return "^" + ans + "$"
 
 
 def glob(metric_names, glob_pattern):
@@ -254,42 +255,122 @@ def glob(metric_names, glob_pattern):
     return list(filter(maybe_matched_prefilter, metric_names))
 
 
-def graphite_glob(accessor, graphite_glob, metrics=True, directories=True):
-    """Get metrics and directories matching a Graphite glob.
+class GlobMetricResult:
+    """Result from a glob search on metrics."""
+
+    def __init__(self, name, value):
+        """Create new instance of GlobMetricResult."""
+        self.name = name
+        self.value = value
+
+    @staticmethod
+    def from_name(name):
+        """Create a new instance of GlobMetricResult from a single name."""
+        return GlobMetricResult(name, None)
+
+    @staticmethod
+    def from_value(value):
+        """Create a new instance GlobMetricResult from a metric."""
+        assert isinstance(value, bg_metric.Metric)
+        return GlobMetricResult(value.name, value)
+
+
+class GlobDirectoryResult:
+    """Result from a glob search on directories."""
+
+    def __init__(self, name):
+        """Create new instance of GlobMetricResult."""
+        self.name = name
+        # value needs to be defined because this class and GlobMetricResult
+        # are meant to be used indifferently
+        self.value = None
+
+    @staticmethod
+    def from_name(name):
+        """Create a new instance of GlobDirectoryResult from a single name."""
+        return GlobDirectoryResult(name)
+
+    def __str__(self):
+        return self.name
+
+
+def graphite_glob(
+    accessor,
+    graphite_glob,
+    metrics=True,
+    directories=True,
+    start_time=None,
+    end_time=None,
+):
+    """Get metric and directory names matching a Graphite glob.
 
     Args:
       accessor: BigGraphite accessor
       graphite_glob: Graphite glob expression
       metrics: True if metrics should be fetched
       directories: True if directories should be fetched
+      start_time: Lower bound of search time range (use None for no bound)
+      end_time: Upper bound of search time range (use None for no bound)
 
     Returns:
       A tuple:
-        First element: sorted list of Cassandra metrics matched by the glob.
-        Second element: sorted list of Cassandra directories matched by the glob.
+        First element: sorted list of metric names matched by the glob.
+        Second element: sorted list of directory names matched by the glob.
     """
     if not _is_valid_glob(graphite_glob):
         # TODO(d.forest): should we instead raise an exception?
         return ([], [])
 
-    glob_re = re.compile(_glob_to_regex(graphite_glob))
-
     if metrics:
-        metrics = accessor.glob_metric_names(graphite_glob)
-        metrics = list(filter(glob_re.match, metrics))
+        metrics = accessor.glob_metric_names(graphite_glob, start_time, end_time)
     else:
         metrics = []
 
     if directories:
-        directories = accessor.glob_directory_names(graphite_glob)
-        directories = list(filter(glob_re.match, directories))
+        directories = accessor.glob_directory_names(graphite_glob, start_time, end_time)
     else:
         directories = []
 
-    return (metrics, directories)
+    return [GlobMetricResult.from_name(metric_name) for metric_name in metrics], \
+           [GlobDirectoryResult.from_name(directory_name) for directory_name in directories]
 
 
-class GlobExpression:
+def graphite_glob_leaves(accessor,
+                         graphite_glob,
+                         start_time=None,
+                         end_time=None):
+    """Get metrics matching a Graphite glob.
+
+    Args:
+      accessor: BigGraphite accessor
+      graphite_glob: Graphite glob expression
+      start_time: Lower bound of search time range (use None for no bound)
+      end_time: Upper bound of search time range (use None for no bound)
+
+    Returns:
+      A tuple:
+        First element: sorted list of metrics matched by the glob.
+        Second element: always empty directory list.
+    """
+    if not _is_valid_glob(graphite_glob):
+        return [], []
+
+    if accessor.TYPE == "cassandra":
+        metrics = accessor.glob_metric_names(graphite_glob, start_time, end_time)
+        results = [GlobMetricResult.from_name(metric) for metric in metrics], []
+    else:
+        metrics = accessor.glob_metrics(graphite_glob, start_time, end_time)
+        results = [GlobMetricResult.from_value(metric) for metric in metrics], []
+    return results
+
+
+def filter_from_glob(names, glob_repr):
+    """Filter metric or directory names from provided glob."""
+    glob_re = re.compile(glob_to_regex(glob_repr))
+    return list(filter(glob_re.match, names))
+
+
+class GlobExpression(object):
     """Base class for glob expressions."""
 
     def __repr__(self):
@@ -305,13 +386,13 @@ class GlobExpressionWithValues(GlobExpression):
     def __init__(self, values):
         """Take a list of values, and stores the sorted unique values."""
         self.values = sorted(set(values))
+        self.negated = False
 
     def __repr__(self):
         return "%s(%s)" % (self.__class__.__name__, self.values)
 
     def __eq__(self, other):
-        return (GlobExpression.__eq__(self, other) and
-                self.values == other.values)
+        return GlobExpression.__eq__(self, other) and self.values == other.values
 
 
 class Globstar(GlobExpression):
@@ -326,6 +407,21 @@ class AnyChar(GlobExpression):
     pass
 
 
+class CharIn(GlobExpressionWithValues):
+    """Represents any single character."""
+
+    pass
+
+
+class CharNotIn(GlobExpressionWithValues):
+    """Represents any single character."""
+
+    def __init__(self, values):
+        """Initializes the CharNotIn."""
+        super(CharNotIn, self).__init__(values)
+        self.negated = True
+
+
 class AnySequence(GlobExpression):
     """Represents any sequence of 0 or more characters."""
 
@@ -346,12 +442,12 @@ class GraphiteGlobParser:
 
     def __init__(self):
         """Build a parser, fill in default values."""
-        self._reset('')
+        self._reset("")
 
     def _commit_sequence(self):
         if len(self._sequence) > 0:
             self._component.append(self._sequence)
-            self._sequence = ''
+            self._sequence = ""
 
     def _commit_component(self):
         self._commit_sequence()
@@ -368,7 +464,7 @@ class GraphiteGlobParser:
         """Parse multi-character wildcard, and globstar."""
         self._commit_sequence()
         # Look-ahead for potential globstar
-        if i < n and self._glob[i] == '*':
+        if i < n and self._glob[i] == "*":
             self._commit_component()
             self._parsed.append(Globstar())
             i += 1
@@ -380,17 +476,35 @@ class GraphiteGlobParser:
     def _find_char_selector_end(self, i, n):
         """Find where a character selector expression ends."""
         j = i
-        if j < n and self._glob[j] == '!':
+        if j < n and self._glob[j] == "!":
             j += 1
-        if j < n and self._glob[j] == ']':
+        if j < n and self._glob[j] == "]":
             j += 1
 
-        j = self._glob.find(']', j)
+        j = self._glob.find("]", j)
         if j == -1:
             return n
 
         return j
 
+    def _group_char_selector(self, chars):
+        """Separate chars from char ranges."""
+        i = 0
+        result = set()
+        size = len(chars)
+        while i < size:
+            if i < size - 2 and chars[i + 1] == "-":
+                # char range.
+                result.add(chars[i:i + 3])
+                i += 3
+            elif chars[i] != "-":
+                result.add(chars[i])
+                i += 1
+            else:
+                # Ignore wrongly positioned -
+                i += 1
+        return result
+
     def _parse_char_selector(self, i, n):
         """Parse character selector, with support for negation and ranges.
 
@@ -400,13 +514,20 @@ class GraphiteGlobParser:
         """
         j = self._find_char_selector_end(i, n)
         if j < n:
+            chars = self._glob[i:j]
+            if chars[0] == "!":
+                chars = self._group_char_selector(chars[1:])
+                char = CharNotIn(chars)
+            else:
+                chars = self._group_char_selector(chars)
+                char = CharIn(chars)
             # +1 to skip closing bracket
             i = j + 1
             self._commit_sequence()
-            self._component.append(AnyChar())
+            self._component.append(char)
         else:
             # Reached end of string: unbalanced bracket
-            self._sequence += '['
+            self._sequence += "["
 
         return i
 
@@ -430,7 +551,7 @@ class GraphiteGlobParser:
                 self._commit_sequence()
                 self._component.append(seq)
         else:
-            self._sequence += '{'
+            self._sequence += "{"
 
         return i
 
@@ -438,17 +559,17 @@ class GraphiteGlobParser:
         has_char_selector = False
         values = []
         curr = []
-        tmp = ''
+        tmp = ""
         j = i
-        c = ''
-        while j < n and c != '}':
+        c = ""
+        while j < n and c != "}":
             c = self._glob[j]
             j += 1
             # Parse sub-expression then combine values with prefixes.
-            if c == '{':
-                if tmp != '':
+            if c == "{":
+                if tmp != "":
                     curr.append(tmp)
-                    tmp = ''
+                    tmp = ""
 
                 result = self._parse_sequence_selector_values(j, n)
                 if not result:
@@ -458,16 +579,16 @@ class GraphiteGlobParser:
                 has_char_selector = has_char_selector or has_charsel
                 curr = [prefix + x for prefix in curr for x in subvalues]
             # End of current element, combine values with suffix.
-            elif c == ',' or c == '}':
+            elif c == "," or c == "}":
                 if len(curr) > 0:
                     values += [x + tmp for x in curr]
                 else:
                     values.append(tmp)
 
                 curr = []
-                tmp = ''
+                tmp = ""
             # Simplified handling of char selector
-            elif c == '[':
+            elif c == "[":
                 # XXX(d.forest): We could keep track of depth and just make sure
                 #                the selector expression is well-formed instead
                 #                of continuing to parse everything.
@@ -477,9 +598,9 @@ class GraphiteGlobParser:
                     has_char_selector = True
                     j = k + 1
                 else:
-                    tmp += '['
+                    tmp += "["
             # Reject dots inside selectors
-            elif c == '.':
+            elif c == ".":
                 return None
             # Append char to the current value.
             else:
@@ -487,7 +608,7 @@ class GraphiteGlobParser:
 
         # We have reached the end without finding a closing brace: the braces
         # are unbalanced, expression cannot be parsed as a sequence selector.
-        if j == n and c != '}':
+        if j == n and c != "}":
             return None
 
         return has_char_selector, j, values + curr
@@ -496,7 +617,7 @@ class GraphiteGlobParser:
         self._glob = glob
         self._parsed = []
         self._component = []
-        self._sequence = ''
+        self._sequence = ""
 
     def parse(self, glob):
         """Parse a graphite glob expression into simple components."""
@@ -507,18 +628,30 @@ class GraphiteGlobParser:
         while i < n:
             c = self._glob[i]
             i += 1
-            if c == '?':
+            if c == "?":
                 self._parse_char_wildcard()
-            elif c == '*':
+            elif c == "*":
                 i = self._parse_wildcard(i, n)
-            elif c == '[':
+            elif c == "[":
                 i = self._parse_char_selector(i, n)
-            elif c == '{':
+            elif c == "{":
                 i = self._parse_sequence_selector(i, n)
-            elif c == '.':
+            elif c == ".":
                 self._commit_component()
             else:
                 self._sequence += c
 
         self._commit_component()
         return self._parsed
+
+    @staticmethod
+    def is_fully_defined(components):
+        """Return True if the components represent a fully-defined metric name."""
+        if Globstar() in components:
+            return False
+        for c in components:
+            if len(c) != 1:
+                return False
+            if not is_fixed_sequence(c[0]):
+                return False
+        return True
