@@ -27,20 +27,32 @@ from __future__ import absolute_import
 from __future__ import print_function
 
 import abc
+import logging
 import os
-from os import path as os_path
+import random
 import sys
 import threading
-import logging
-import uuid
 import time
-import random
+import uuid
+from os import path as os_path
 
 import cachetools
 import lmdb
+import prometheus_client
 import six
 
-from biggraphite import accessor as bg_accessor
+from biggraphite import metric as bg_metric
+
+METRICS_LABELS = ["type", "name"]
+
+CACHE_SIZE = prometheus_client.Gauge("bg_cache_size", "Current size", METRICS_LABELS)
+CACHE_MAX_SIZE = prometheus_client.Gauge("bg_cache_maxsize", "Max size", METRICS_LABELS)
+CACHE_HITS = prometheus_client.Counter(
+    "bg_cache_hits_total", "Cache hits", METRICS_LABELS
+)
+CACHE_MISSES = prometheus_client.Counter(
+    "bg_cache_misses_total", "Cache misses", METRICS_LABELS
+)
 
 
 class Error(Exception):
@@ -55,12 +67,11 @@ class MetadataCache(object):
     """A metadata cache."""
 
     __metaclass__ = abc.ABCMeta
+    TYPE = "abstract"
 
-    def __init__(self, accessor, settings):
+    def __init__(self, accessor, settings, name=None):
         """Create a new DiskCache."""
         assert accessor
-        self.hit_count = 0
-        self.miss_count = 0
         self._lock = threading.Lock()
         self._accessor_lock = threading.Lock()
         self._accessor = accessor
@@ -70,6 +81,16 @@ class MetadataCache(object):
         self._json_cache_lock = threading.Lock()
         self._json_cache = {}
 
+        if name is None:
+            name = str(hash(self))
+        self.name = name
+
+        self._size = CACHE_SIZE.labels(self.TYPE, name)
+        self._size.set_function(lambda: self.stats()["size"])
+        self._max_size = CACHE_MAX_SIZE.labels(self.TYPE, name)
+        self._hits = CACHE_HITS.labels(self.TYPE, name)
+        self._misses = CACHE_MISSES.labels(self.TYPE, name)
+
     @abc.abstractmethod
     def open(self):
         """Allocate ressources used by the cache.
@@ -94,18 +115,29 @@ class MetadataCache(object):
         """
         pass
 
+    @property
+    def hit_count(self):
+        """Hit counter."""
+        return self._hits._value.get()
+
+    @property
+    def miss_count(self):
+        """Miss counter."""
+        return self._misses._value.get()
+
     def cache_has(self, metric_name):
         """Check if a metric is cached."""
-        return self._cache_has(metric_name)
+        hit = self._cache_has(metric_name)
+        if hit:
+            self._hits.inc()
+        else:
+            self._misses.inc()
+        return hit
 
     def cache_set(self, metric_name, metric):
         """Insert a metric in the cache."""
         return self._cache_set(metric_name, metric)
 
-    def make_metric(self, name, metadata):
-        """Create a metric object from a name and metadata."""
-        return self._accessor.make_metric(name, metadata)
-
     def create_metric(self, metric, metric_name=None):
         """Create a metric definition from a Metric."""
         metric_name = metric_name or metric.name
@@ -130,17 +162,17 @@ class MetadataCache(object):
                 self._cache_set(metric_name, metric)
         return found
 
-    def get_metric(self, metric_name, touch=False):
+    def get_metric(self, metric_name):
         """Return a Metric for this metric_name, None if no such metric."""
         metric, hit = self._cache_get(metric_name)
         if hit:
-            self.hit_count += 1
+            self._hits.inc()
         else:
-            self.miss_count += 1
+            self._misses.inc()
         # Check that is still doesn't exists.
         if not metric:
             with self._accessor_lock:
-                metric = self._accessor.get_metric(metric_name, touch=touch)
+                metric = self._accessor.get_metric(metric_name)
                 self._cache_set(metric_name, metric)
 
         return metric
@@ -151,7 +183,14 @@ class MetadataCache(object):
         pass
 
     @abc.abstractmethod
-    def repair(self, start_key=None, end_key=None, shard=0, nshards=1, callback_on_progress=None):
+    def repair(
+        self,
+        start_key=None,
+        end_key=None,
+        shard=0,
+        nshards=1,
+        callback_on_progress=None,
+    ):
         """Remove spurious entries from the cache.
 
         During the repair the keyspace is split in nshards and
@@ -172,7 +211,7 @@ class MetadataCache(object):
         with self._json_cache_lock:
             metadata = self._json_cache.get(metadata_str)
             if not metadata:
-                metadata = bg_accessor.MetricMetadata.from_json(metadata_str)
+                metadata = bg_metric.MetricMetadata.from_json(metadata_str)
                 self._json_cache[metadata_str] = metadata
         return metadata
 
@@ -191,27 +230,40 @@ class MetadataCache(object):
         """Put metric in the cache."""
         pass
 
+    @abc.abstractmethod
+    def stats(self):
+        """Current stats about the cache the cache."""
+        return {"size": 0, "maxsize": 0, "hits": 0, "miss": 0}
+
 
 class MemoryCache(MetadataCache):
     """A per-process memory cache."""
 
-    def __init__(self, accessor, settings):
+    TYPE = "memory"
+
+    def __init__(self, accessor, settings, name=None):
         """Initialize the memory cache."""
-        super(MemoryCache, self).__init__(accessor, settings)
-        self.__size = settings.get('size', 1 * 1000 * 1000)
-        self.__ttl = int(settings.get('ttl', 24 * 60 * 60))
+        super(MemoryCache, self).__init__(accessor, settings, name)
+        self.__size = settings.get("size", 1 * 1000 * 1000)
+        self.__ttl = int(settings.get("ttl", 24 * 60 * 60))
+        self._max_size.set(self.__size)
 
     def open(self):
         """Allocate ressources used by the cache."""
+        super(MemoryCache, self).open()
+
         def _timer():
             # Use a custom timer to try to spread expirations. Within one instance it
             # won't change anything but it will be better if you run multiple instances.
             return time.time() + self.__ttl * random.uniform(-0.25, 0.25)
+
         self.__cache = cachetools.TTLCache(
-            maxsize=self.__size, ttl=self.__ttl, timer=_timer)
+            maxsize=self.__size, ttl=self.__ttl, timer=_timer
+        )
 
     def close(self):
         """Free resources allocated by open()."""
+        super(MemoryCache, self).close()
         self.__cache = None
 
     def _cache_has(self, metric_name):
@@ -242,11 +294,17 @@ class MemoryCache(MetadataCache):
         with self._lock:
             self.__cache.expire()
 
-    def repair(self, start_key=None, end_key=None, shard=0, nshards=1, callback_on_progress=None):
+    def repair(
+        self,
+        start_key=None,
+        end_key=None,
+        shard=0,
+        nshards=1,
+        callback_on_progress=None,
+    ):
         """Remove spurious entries from the cache."""
         with self._lock:
-            self._repair(start_key, end_key, shard,
-                         nshards, callback_on_progress)
+            self._repair(start_key, end_key, shard, nshards, callback_on_progress)
 
     def _repair(self, start_key, end_key, shard, nshards, callback_on_progress):
         i = 0
@@ -263,8 +321,9 @@ class MemoryCache(MetadataCache):
             expected_metric = self.__cache[key]
             if metric != expected_metric:
                 logging.warning(
-                    "Removing invalid key '%s': expected: %s cached: %s" % (
-                        key, expected_metric, metric))
+                    "Removing invalid key '%s': expected: %s cached: %s"
+                    % (key, expected_metric, metric)
+                )
                 del self.__cache[key]
 
             if callback_on_progress:
@@ -272,6 +331,18 @@ class MemoryCache(MetadataCache):
                 total = end_key - start_key if start_key and end_key else None
                 callback_on_progress(done, total)
 
+    def stats(self):
+        """Current stats about the cache the cache."""
+        if self.__cache:
+            return {
+                    "size": self.__cache.currsize,
+                    "maxsize": self.__cache.maxsize,
+                    "hits": self._hits._value.get(),
+                    "miss": self._misses._value.get(),
+                }
+        else:
+            return super(MemoryCache, self).stats()
+
 
 class DiskCache(MetadataCache):
     """A metadata cache that can be shared between processes trusting each other.
@@ -280,6 +351,8 @@ class DiskCache(MetadataCache):
     See module-level comments for the design.
     """
 
+    TYPE = "disk"
+
     __SINGLETONS = {}
     __SINGLETONS_LOCK = threading.Lock()
     # Maximum number of concurrent readers.
@@ -287,17 +360,16 @@ class DiskCache(MetadataCache):
     # According to LMDB's author, 128 readers is about 8KiB of RAM, 1024 is about 128kiB and even
     # 4096 is safe: https://twitter.com/armon/status/534867803426533376
     _MAX_READERS = 2048
-    _METRIC_SEPARATOR = '|'
-    _EMPTY = b'nil'
+    _METRIC_SEPARATOR = "|"
+    _EMPTY = b"nil"
 
     # 1G on 32 bits systems
     # 16G on 64 bits systems
-    MAP_SIZE = (1024 * 1024 * 1024 * 16 if sys.maxsize >
-                2**32 else 1024 * 1024 * 1024)
+    MAP_SIZE = 1024 * 1024 * 1024 * 16 if sys.maxsize > 2 ** 32 else 1024 * 1024 * 1024
 
-    def __init__(self, accessor, settings):
+    def __init__(self, accessor, settings, name=None):
         """Create a new DiskCache."""
-        super(DiskCache, self).__init__(accessor, settings)
+        super(DiskCache, self).__init__(accessor, settings, name)
 
         path = settings.get("path")
         assert path
@@ -307,10 +379,9 @@ class DiskCache(MetadataCache):
         self.__size = settings.get("size", self.MAP_SIZE)
         self.__ttl = int(settings.get("ttl", 24 * 60 * 60))
         self.__sync = settings.get("sync", True)
-        self.__databases = {
-            "metric_to_meta": None
-        }
+        self.__databases = {"metric_to_meta": None}
         self.__metric_to_metadata_db = None
+        self._max_size.set(self.__size)
 
     def open(self):
         """Allocate ressources used by the cache.
@@ -325,8 +396,9 @@ class DiskCache(MetadataCache):
         except OSError:
             pass  # Directory already exists
 
-        logging.info('Opening cache %s (ttl: %s, sync: %s)',
-                     self.__path, self.__ttl, self.__sync)
+        logging.info(
+            "Opening cache %s (ttl: %s, sync: %s)", self.__path, self.__ttl, self.__sync
+        )
         self.__env = lmdb.open(
             self.__path,
             map_size=self.__size,
@@ -368,7 +440,7 @@ class DiskCache(MetadataCache):
         if self.__env:
             self.__env.close()
             self.__env = None
-            super(DiskCache, self).close()
+        super(DiskCache, self).close()
 
     def _cache_has(self, metric_name):
         """Check if metric is cached."""
@@ -390,12 +462,12 @@ class DiskCache(MetadataCache):
             return value
         if isinstance(value, six.binary_type):
             return value
-        return value.encode('utf-8')
+        return value.encode("utf-8")
 
     def _decode(self, value):
         if value is None:
             return value
-        return value.decode('utf-8')
+        return value.decode("utf-8")
 
     def _cache_get(self, metric_name):
         """Return a Metric from a the cache, None if no such metric."""
@@ -440,7 +512,7 @@ class DiskCache(MetadataCache):
                 txn.delete(key=encoded_metric_name)
 
         metadata = self.metadata_from_str(metadata_str)
-        return bg_accessor.Metric(metric_name, id, metadata), True
+        return bg_metric.Metric(metric_name, id, metadata), True
 
     def _cache_set(self, metric_name, metric):
         """If metric is valid, add it to the cache.
@@ -456,13 +528,14 @@ class DiskCache(MetadataCache):
         with self.__env.begin(self.__metric_to_metadata_db, write=True) as txn:
             txn.put(key, value, dupdata=False, overwrite=True)
 
-    def stat(self):
+    def stats(self):
         """Count number of cached entries."""
-        ret = {}
-        ret[''] = self.__env.stat(),
+        ret = super(DiskCache, self).stats()
+        ret["root"] = (self.__env.stat(),)
         for name, database in self.__databases.items():
             with self.__env.begin(database, write=False) as txn:
                 ret[name] = txn.stat(database)
+
         return ret
 
     def __value_from_strings(self, metric_id, metric_metadata):
@@ -477,8 +550,7 @@ class DiskCache(MetadataCache):
         """
         timestamp = str(int(time.time()))
         return self._encode(
-            self._METRIC_SEPARATOR.join(
-                [metric_id, metric_metadata, timestamp])
+            self._METRIC_SEPARATOR.join([metric_id, metric_metadata, timestamp])
         )
 
     def __value_from_metric(self, metric):
@@ -540,19 +612,27 @@ class DiskCache(MetadataCache):
             split = self.__split_payload(self._decode(value))
             if split is None:
                 logging.warning(
-                    "Removing undecodable key '%s' with value %s" % (key, value))
+                    "Removing undecodable key '%s' with value %s" % (key, value)
+                )
                 txn.delete(key=key)
                 continue
 
             _, _, timestamp = split
             if timestamp < cutoff:
                 logging.warning(
-                    "Removing expired key '%s' with timestamp %d" % (
-                        key, timestamp))
+                    "Removing expired key '%s' with timestamp %d" % (key, timestamp)
+                )
                 txn.delete(key=key)
         return None
 
-    def repair(self, start_key=None, end_key=None, shard=0, nshards=1, callback_on_progress=None):
+    def repair(
+        self,
+        start_key=None,
+        end_key=None,
+        shard=0,
+        nshards=1,
+        callback_on_progress=None,
+    ):
         """Remove spurious entries from the cache.
 
         During the repair the keyspace is split in nshards and
@@ -581,24 +661,65 @@ class DiskCache(MetadataCache):
                     continue
 
                 metric = self._accessor.get_metric(self._decode(key))
-                expected_value = self.__value_from_metric(
-                    metric) if metric else None
+                expected_value = self.__value_from_metric(metric) if metric else None
 
                 split = self.__split_payload(self._decode(value))
-                v_id, v_metadata, _ = split if split is not None else (
-                    None, None, None)
+                v_id, v_metadata, _ = split if split is not None else (None, None, None)
 
                 split = self.__split_payload(self._decode(expected_value))
-                e_id, e_metadata, _ = split if split is not None else (
-                    None, None, None)
+                e_id, e_metadata, _ = split if split is not None else (None, None, None)
 
-                if v_id is None or e_id is None or (v_id, v_metadata) != (e_id, e_metadata):
+                if (
+                    v_id is None
+                    or e_id is None
+                    or (v_id, v_metadata) != (e_id, e_metadata)
+                ):
                     logging.warning(
-                        "Removing invalid key '%s': expected: %s cached: %s" % (
-                            key, expected_value, value))
+                        "Removing invalid key '%s': expected: %s cached: %s"
+                        % (key, expected_value, value)
+                    )
                     txn.delete(key=key)
 
                 if callback_on_progress:
                     done = key - start_key if start_key else key
                     total = end_key - start_key if start_key and end_key else None
                     callback_on_progress(done, total)
+
+
+class NoneCache(MetadataCache):
+    """Dummy metadata cache used when no cache is required."""
+
+    def __init__(self, accessor, settings, name=None):
+        """Create a new NoneCache."""
+        super(NoneCache, self).__init__(accessor, settings, name)
+        assert accessor
+        self._accessor = accessor
+
+    def open(self):
+        """No resource has to be allocated for this cache. Does nothing."""
+        super(NoneCache, self).open()
+
+    def close(self):
+        """No resource have to be freed for this cache. Does nothing."""
+        super(NoneCache, self).close()
+
+    def clean(self):
+        """Cleaning this cache is useless. Does nothing."""
+        super(NoneCache, self).clean()
+
+    def repair(self, start_key=None, end_key=None, shard=0, nshards=1, callback_on_progress=None):
+        """Repairing this cache is useless. Does nothing."""
+        super(NoneCache, self).repair()
+
+    def _cache_has(self, metric_name):
+        return True
+
+    def _cache_get(self, metric_name):
+        return self._accessor.get_metric(metric_name), True
+
+    def _cache_set(self, metric_name, metric):
+        pass
+
+    def stats(self):
+        """Stats are not computed for this cache."""
+        return super(NoneCache, self).stats()
